mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: Milian Wolff <milian.wolff@kdab.com>
To: acme@kernel.org, jolsa@kernel.org, Jin Yao <yao.jin@linux.intel.com>
Cc: Linux-kernel@vger.kernel.org, linux-perf-users@vger.kernel.org,
	Milian Wolff <milian.wolff@kdab.com>,
	Arnaldo Carvalho de Melo <acme@redhat.com>,
	David Ahern <dsahern@gmail.com>,
	Namhyung Kim <namhyung@kernel.org>,
	Peter Zijlstra <a.p.zijlstra@chello.nl>
Subject: [PATCH v3 09/13] perf report: properly handle branch count in match_chain
Date: Wed,  6 Sep 2017 15:54:57 +0200	[thread overview]
Message-ID: <20170906135501.306-10-milian.wolff@kdab.com> (raw)
In-Reply-To: <20170906135501.306-1-milian.wolff@kdab.com>

Some of the code paths I introduced before returned too early
without running the code to handle a node's branch count.
By refactoring match_chain to only have one exit point, this
can be remedied.

Cc: Arnaldo Carvalho de Melo <acme@redhat.com>
Cc: David Ahern <dsahern@gmail.com>
Cc: Namhyung Kim <namhyung@kernel.org>
Cc: Peter Zijlstra <a.p.zijlstra@chello.nl>
Cc: Yao Jin <yao.jin@linux.intel.com>
Signed-off-by: Milian Wolff <milian.wolff@kdab.com>
---
 tools/perf/util/callchain.c | 115 +++++++++++++++++++++++---------------------
 1 file changed, 59 insertions(+), 56 deletions(-)

diff --git a/tools/perf/util/callchain.c b/tools/perf/util/callchain.c
index 5c8aaa2c0123..9c355f02934f 100644
--- a/tools/perf/util/callchain.c
+++ b/tools/perf/util/callchain.c
@@ -659,77 +659,80 @@ static enum match_result match_chain_strings(const char *left,
 	return ret;
 }
 
+static enum match_result match_address(u64 left, u64 right)
+{
+	if (left == right)
+		return MATCH_EQ;
+	else if (left < right)
+		return MATCH_LT;
+	else
+		return MATCH_GT;
+}
+
 static enum match_result match_chain(struct callchain_cursor_node *node,
 				     struct callchain_list *cnode)
 {
-	struct symbol *sym = node->sym;
-	enum match_result match;
-	u64 left, right;
+	enum match_result match = MATCH_ERROR;
 
-	if (callchain_param.key == CCKEY_SRCLINE) {
+	switch (callchain_param.key) {
+	case CCKEY_SRCLINE:
 		match = match_chain_strings(cnode->srcline, node->srcline);
-
-		// if no srcline is available, fallback to symbol name
-		if (match == MATCH_ERROR && cnode->ms.sym && node->sym)
-			match = match_chain_strings(cnode->ms.sym->name,
-						    node->sym->name);
-
 		if (match != MATCH_ERROR)
-			return match;
-
-		// otherwise fall-back to IP-based comparison below
-	}
-
-	if (cnode->ms.sym && sym && callchain_param.key == CCKEY_FUNCTION) {
-		// compare inlined frames based on their symbol name because
-		// different inlined frames will have the same symbol start
-		if (cnode->ms.sym->inlined || node->sym->inlined)
-			return match_chain_strings(cnode->ms.sym->name,
-						   node->sym->name);
-
-		left = cnode->ms.sym->start;
-		right = sym->start;
-	} else {
-		left = cnode->ip;
-		right = node->ip;
+			break;
+		// else fall-through
+	case CCKEY_FUNCTION:
+		if (node->sym && cnode->ms.sym) {
+			// compare inlined frames based on their symbol name
+			// because different inlined frames will have the same
+			// symbol start. otherwise do a faster comparison based
+			// on the symbol start address
+			if (cnode->ms.sym->inlined || node->sym->inlined)
+				match = match_chain_strings(cnode->ms.sym->name,
+							    node->sym->name);
+			else
+				match = match_address(cnode->ms.sym->start,
+						      node->sym->start);
+			if (match != MATCH_ERROR)
+				break;
+		}
+		// else fall-through
+	case CCKEY_ADDRESS:
+	default:
+		match = match_address(cnode->ip, node->ip);
+		break;
 	}
 
-	if (left == right) {
-		if (node->branch) {
-			cnode->branch_count++;
+	if (match == MATCH_EQ && node->branch) {
+		cnode->branch_count++;
 
-			if (node->branch_from) {
-				/*
-				 * It's "to" of a branch
-				 */
-				cnode->brtype_stat.branch_to = true;
+		if (node->branch_from) {
+			/*
+			 * It's "to" of a branch
+			 */
+			cnode->brtype_stat.branch_to = true;
 
-				if (node->branch_flags.predicted)
-					cnode->predicted_count++;
+			if (node->branch_flags.predicted)
+				cnode->predicted_count++;
 
-				if (node->branch_flags.abort)
-					cnode->abort_count++;
+			if (node->branch_flags.abort)
+				cnode->abort_count++;
 
-				branch_type_count(&cnode->brtype_stat,
-						  &node->branch_flags,
-						  node->branch_from,
-						  node->ip);
-			} else {
-				/*
-				 * It's "from" of a branch
-				 */
-				cnode->brtype_stat.branch_to = false;
-				cnode->cycles_count +=
-					node->branch_flags.cycles;
-				cnode->iter_count += node->nr_loop_iter;
-				cnode->iter_cycles += node->iter_cycles;
-			}
+			branch_type_count(&cnode->brtype_stat,
+					  &node->branch_flags,
+					  node->branch_from,
+					  node->ip);
+		} else {
+			/*
+			 * It's "from" of a branch
+			 */
+			cnode->brtype_stat.branch_to = false;
+			cnode->cycles_count += node->branch_flags.cycles;
+			cnode->iter_count += node->nr_loop_iter;
+			cnode->iter_cycles += node->iter_cycles;
 		}
-
-		return MATCH_EQ;
 	}
 
-	return left > right ? MATCH_GT : MATCH_LT;
+	return match;
 }
 
 /*
-- 
2.14.1

  parent reply	other threads:[~2017-09-06 13:57 UTC|newest]

Thread overview: 23+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2017-09-06 13:54 [PATCH v3 00/13] generate full callchain cursor entries for inlined frames Milian Wolff
2017-09-06 13:54 ` [PATCH v3 01/13] perf report: remove code to handle inline frames from browsers Milian Wolff
2017-09-18 11:56   ` Jiri Olsa
2017-09-18 12:43     ` Milian Wolff
2017-09-06 13:54 ` [PATCH v3 02/13] perf util: store srcline in callchain_cursor_node Milian Wolff
2017-09-06 13:54 ` [PATCH v3 03/13] perf report: create real callchain entries for inlined frames Milian Wolff
2017-09-19 12:27   ` Jiri Olsa
2017-10-01 12:37     ` Milian Wolff
2017-10-01 17:40       ` Joe Perches
2017-09-19 12:27   ` Jiri Olsa
2017-10-01 14:12     ` Milian Wolff
2017-09-19 12:27   ` Jiri Olsa
2017-10-01 12:40     ` Milian Wolff
2017-09-06 13:54 ` [PATCH v3 04/13] perf report: fall-back to function name comparison for -g srcline Milian Wolff
2017-09-06 13:54 ` [PATCH v3 05/13] perf report: mark inlined frames in output by " (inlined)" suffix Milian Wolff
2017-09-06 13:54 ` [PATCH v3 06/13] perf script: mark inlined frames and do not print DSO for them Milian Wolff
2017-09-06 13:54 ` [PATCH v3 07/13] perf report: compare symbol name for inlined frames when matching Milian Wolff
2017-09-06 13:54 ` [PATCH v3 08/13] perf report: compare symbol name for inlined frames when sorting Milian Wolff
2017-09-06 13:54 ` Milian Wolff [this message]
2017-09-06 13:54 ` [PATCH v3 10/13] perf report: cache failed lookups of inlined frames Milian Wolff
2017-09-06 13:54 ` [PATCH v3 11/13] perf report: cache srclines for callchain nodes Milian Wolff
2017-09-06 13:55 ` [PATCH v3 12/13] perf report: use srcline from callchain for hist entries Milian Wolff
2017-09-06 13:55 ` [PATCH v3 13/13] perf util: enable handling of inlined frames by default Milian Wolff

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20170906135501.306-10-milian.wolff@kdab.com \
    --to=milian.wolff@kdab.com \
    --cc=Linux-kernel@vger.kernel.org \
    --cc=a.p.zijlstra@chello.nl \
    --cc=acme@kernel.org \
    --cc=acme@redhat.com \
    --cc=dsahern@gmail.com \
    --cc=jolsa@kernel.org \
    --cc=linux-perf-users@vger.kernel.org \
    --cc=namhyung@kernel.org \
    --cc=yao.jin@linux.intel.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®